// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file MultiGridWrapper.h */

#ifndef AMDIS_MULTIGRIDWRAPPER_H
#define AMDIS_MULTIGRIDWRAPPER_H

#include "MultiGridSolver.h"
#include "MatVecMultiplier.h"
#include "OEMSolver.h"
#include "MemoryManager.h"

namespace AMDiS {

  /** \brief
   * Wrapper to make the multigrid solver accessible as a normal scal solver.
   */
  class MultiGridWrapperScal : public OEMSolver<DOFVector<double> >
  {
  public:
    MEMORY_MANAGED(MultiGridWrapperScal);

    /** \brief
     * Creator class used in the OEMSolverMap.
     */
    class Creator : public OEMSolverCreator<DOFVector<double> >
    {
    public:
      MEMORY_MANAGED(Creator);

      virtual ~Creator() {};

      /** \brief
       * Returns a new object.
       */
      OEMSolver<DOFVector<double> >* create() { 
	return 
	  NEW MultiGridWrapperScal(name);
      };
    };

    MultiGridWrapperScal(std::string name) 
      : OEMSolver<DOFVector<double> >(name)
    {
      mgSolver_ = 
	NEW MultiGridSolverScal(name);
    };

    ~MultiGridWrapperScal() {
      DELETE mgSolver_;
    };

    void init() {};

    void exit() {};

    int solveSystem(MatVecMultiplier<DOFVector<double> > *mv,
		    DOFVector<double>                    *x, 
		    DOFVector<double>                    *b,
		    bool reuseMatrix)
    {
      DOFMatrix *matrix = 
	dynamic_cast<StandardMatVec<DOFMatrix, DOFVector<double> >*>(mv)->getMatrix();
      return mgSolver_->solve(*matrix, *x, *b, reuseMatrix);
    };

  protected:
    MultiGridSolverBase<DOFMatrix, DOFVector<double> > *mgSolver_;
  };


  /** \brief
   * Wrapper to make the multigrid solver accessible as a normal vector solver.
   */
  class MultiGridWrapperVec : public OEMSolver<SystemVector>
  {
  public:
    MEMORY_MANAGED(MultiGridWrapperVec);

    /** \brief
     * Creator class used in the OEMSolverMap.
     */
    class Creator : public OEMSolverCreator<SystemVector>
    {
    public:
      MEMORY_MANAGED(Creator);

      virtual ~Creator() {};

      /** \brief
       * Returns a new object.
       */
      OEMSolver<SystemVector>* create() { 
	return NEW MultiGridWrapperVec(name);
      };
    };

    MultiGridWrapperVec(std::string name) 
      : OEMSolver<SystemVector>(name)
    {
      mgSolver_ = 
	NEW MultiGridSolverVec(name);
    };

    ~MultiGridWrapperVec() {
      DELETE mgSolver_;
    };

    void init() {};

    void exit() {};

    int solveSystem(MatVecMultiplier<SystemVector> *mv,
		    SystemVector                   *x, 
		    SystemVector                   *b,
		    bool reuseMatrix)
    {
      Matrix<DOFMatrix*> *matrix = 
	dynamic_cast<StandardMatVec<Matrix<DOFMatrix*>, SystemVector>*>(mv)->getMatrix();
      return mgSolver_->solve(*matrix, *x, *b, reuseMatrix);
    };

  protected:
    MultiGridSolverBase<Matrix<DOFMatrix*>, SystemVector> *mgSolver_;
  };

}

#endif
