#include "DOFVector.h"
#include "FiniteElemSpace.h"

namespace AMDiS {

template<typename T>
void SparseVector<T>::refineInterpol(RCNeighbourList& list, int n)
{}

template<typename T>
void SparseVector<T>::coarseRestrict(RCNeighbourList& list, int n)
{}

template<typename T>
void SparseVector<T>::copyToDOFVector(DOFVector<T> &dofVec,
				   std::set<DegreeOfFreedom> &dofs)
{
  std::set<DegreeOfFreedom>::iterator dofsIt;
  std::set<DegreeOfFreedom>::iterator dofsEnd = dofs.end();

  for(dofsIt = dofs.begin(); dofsIt != dofsEnd; ++dofsIt) {
    dofVec[*dofsIt] = operator[](*dofsIt);
  }
}

template<typename T>
void SparseVector<T>::copyToDOFVector(DOFVector<T> &dofVec)
{
  int i, size = static_cast<int>((*physToLog_).size());

//   vec_.resize(size, 0.0);

  for(i = 0; i < size; i++) {
    dofVec[(*physToLog_)[i]] = vec_[i];
  }
}

template<typename T>
void SparseVector<T>::copyFromDOFVector(DOFVector<T>         &dofVec,
				     std::set<DegreeOfFreedom> &dofs)
{
  std::set<DegreeOfFreedom>::iterator dofsIt;
  std::set<DegreeOfFreedom>::iterator dofsEnd = dofs.end();

  for(dofsIt = dofs.begin(); dofsIt != dofsEnd; ++dofsIt) {
    operator[](*dofsIt) = dofVec[*dofsIt];
  }
}

template<typename T>
T& SparseVector<T>::operator[](DegreeOfFreedom i)
{
  int physIndex = (*logToPhys_)[i];

//   vec_.resize(physToLog_->size(), 0.0);

  if(physIndex == 0) {
    vec_.push_back(0.0);
    (*physToLog_).push_back(i);
    physIndex = (*logToPhys_)[i] = static_cast<int>(vec_.size());

    typename std::vector<SparseVector<T>*>::iterator clientIt, clientEnd;
    if(master_) {
      clientEnd = master_->clients_.end();
      master_->vec_.resize(physToLog_->size(), 0.0);
      for(clientIt = master_->clients_.begin(); clientIt != clientEnd; ++clientIt) {
	(*clientIt)->vec_.resize(physToLog_->size(), 0.0);
      }
    } else {
      clientEnd = clients_.end();
      for(clientIt = clients_.begin(); clientIt != clientEnd; ++clientIt) {
	(*clientIt)->vec_.resize(physToLog_->size(), 0.0);
      }      
    }
  }

  return vec_[physIndex-1];
}

template<typename T>
const T& SparseVector<T>::operator[](DegreeOfFreedom i) const
{
  return (*const_cast<SparseVector*>(this))[i];
}

template<typename T>
void SparseVector<T>::clear(bool eraseEntries)
{
//   if(master_ && eraseEntries) {
//     (*logToPhys_).clear();
//     (*physToLog_).resize(0);
//     vec_.resize(0);
//   } else {
//   vec_.resize(physToLog_->size(), 0.0);
  typename std::vector<T>::iterator vecIt, vecEnd = vec_.end();
  for(vecIt = vec_.begin(); vecIt != vecEnd; ++vecIt) {
    *vecIt = 0;
  }
//   }
}

template<typename T>
void SparseVector<T>::copy(SparseVector<T> &x, 
			   const std::set<DegreeOfFreedom> &dofSet)
{
  std::set<DegreeOfFreedom>::iterator dofIt, dofEnd = dofSet.end();
  for(dofIt = dofSet.begin(); dofIt != dofEnd; ++dofIt) {
    operator[](*dofIt) = x[*dofIt];
  }  
}

template<typename T>
void SparseVector<T>::copy(SparseVector<T> &x, 
			   const std::vector<DegreeOfFreedom> &dofSet)
{
  std::vector<DegreeOfFreedom>::iterator dofIt, 
    dofEnd = const_cast< std::vector<DegreeOfFreedom>&>(dofSet).end();
  for(dofIt = const_cast< std::vector<DegreeOfFreedom>&>(dofSet).begin(); 
      dofIt != dofEnd; 
      ++dofIt) 
  {
    operator[](*dofIt) = x[*dofIt];
  }  
}

} 
