#include "demangle.h"
#include <typeinfo>
#include <math.h>
#include <stdio.h>
#include <cstring>
#include "Global.h"

namespace AMDiS {

void demangle_rec(char**, char**);

void demangle(const char* in, char* result)
{
  //printf("mangled name: %s    ", in);
  char** inptr = const_cast<char**>(&in);
  char** resultptr = &result;

  demangle_rec(inptr, resultptr);
  sprintf((*resultptr), "\0");
}

void demangle_template(char **in, char **result)
{
  demangle_rec(in, result);

  sprintf((*result), "<");
  (*result)++;

  // template parameters
  char c = (*in)[0];

  if((c >= '0') && c <= '9') {
    int n;
    sscanf((*in), "%d", &n);
    (*in) += (static_cast<int>(log10(n))) + 1;
    for(int i=0; i < n; i++) {
      if((*in)[0] == 'Z') {
	(*in)++;
	demangle_rec(in, result);
      } else {
	demangle_rec(in, result);
	sprintf((*result), " = ");
	(*result) += 3;
	if((*in)[0] == 'm') {
	  (*in)++;
	}
 	int n;
 	sscanf((*in), "%d", &n);
 	sprintf((*result), "%d", n);
 	(*in) += static_cast<int>(log10(n)) + 1;
 	(*result) += static_cast<int>(log10(n)) + 1;
      }
      if(i != n-1) {
	sprintf((*result), ", ");  
	(*result) += 2;
      }
    }      
  } else {
    ERROR_EXIT("number expected\n");
  }

  sprintf((*result), ">");
  (*result)++;
}

void demangle_qualified(char** in, char** result)
{
  char c = (*in)[0];

  if((c >= '0') && c <= '9') {
    int n = static_cast<int>(c - '0');
    (*in)++;
    for(int i=0; i < n; i++) {
      demangle_rec(in, result);
      if(i != n-1) {
	sprintf((*result), "::");  
	(*result) += 2;
      }
    }      
  } else {
    ERROR_EXIT("number expected\n");
  }
};

void demangle_rec(char** in, char** result)
{
  char c;

  c = (*in)[0];

  if((c >= '0') && (c <= '9')) {
    int l;
    // type name
    sscanf((*in), "%d", &l);
    (*in) += (static_cast<int>(log10(l)) + 1);
    strncpy((*result), (*in), l);
    (*in) += l;
    (*result) += l;    
  } else {
    (*in)++;
    switch(c) {
    case 'A': // array
      break;
    case 'b': // bool
      sprintf((*result), "bool");
      (*result) += 4;
      break;
    case 'c': // char
      sprintf((*result), "char");
      (*result) += 4;
      break;
    case 'C': // const 
      sprintf((*result), "const ");
      (*result) += 6;
      demangle_rec(in, result);
      break;
    case 'd': // double
      sprintf((*result), "double");
      (*result) += 6;
      break;
    case 'f': // float
      sprintf((*result), "float");
      (*result) += 5;
      break;
    case 'i': // int
      sprintf((*result), "int");
      (*result) += 3;    
      break;
    case 'J': // complex type
      break;
    case 'l': // long
      break;
    case 'P': // pointer
      demangle_rec(in, result);
      sprintf((*result), "*");
      (*result)++;
      break;
    case 'Q': // qualified name
      demangle_qualified(in, result);
      break;
    case 'r': // long double
      break;
    case 'R': // reference
      demangle_rec(in, result);
      sprintf((*result), "&");
      (*result)++;
      break;
    case 's': // short
      break;
    case 'S': // signed
      sprintf((*result), "signed ");
      (*result) += 7;
      demangle_rec(in, result);      
      break;
    case 't': // template
      demangle_template(in, result);
      break;
    case 'U': // unsigned
      sprintf((*result), "unsigned ");
      (*result) += 9;
      demangle_rec(in, result);      
      break;
    case 'v': // void
      break;
    case 'x': // long long
      break;
    default:
      ;
      //instream.unget();
    }
  }
}

}
